package com.persagy.job;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.client.GroupNettyClient;
import com.persagy.entity.AlarmState;
import com.persagy.entity.NettyMessage;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.utils.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.util.Arrays;
import java.util.Date;
import java.util.concurrent.atomic.AtomicLong;

import static org.quartz.SimpleScheduleBuilder.simpleSchedule;

@DisallowConcurrentExecution
@PersistJobDataAfterExecution
@Slf4j
public class AlarmExpireJob extends QuartzJobBean {

    @Autowired
    private GroupNettyClient groupNettyClient;
    @Autowired
    AlarmRecordRepository alarmRecordRepository;
    @Autowired
    AlarmInfoCache alarmInfoCache;
    /**
     * 报警记录信息详情
     */
    private String alarmRecord;
    /**
     * 重试次数
     */
    private String refire;
    /**
     * 过期时间
     */
    private String expireTime;
    /**
     * 报警定义标识（itemcode+objId）
     */
    private String defineId;
    /**
     * 集团编码
     */
    @Value("${group.code}")
    public String groupCode;

    /**
     * 项目名称
     */
    @Value("${project.id}")
    String projectId;

    private AtomicLong nums = new AtomicLong(1L);

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        try {
            JobDataMap mergedJobDataMap = context.getMergedJobDataMap();
            log.info("----------------开始---------------------");
            log.warn("refireCount:[{}],应该执行时间：[{}],实际执行时间：[{}]",context.getRefireCount(),expireTime,DateUtil.format(context.getFireTime(), DatePattern.ISO8601_PATTERN));
            if(StringUtils.isNotBlank(alarmRecord)) {
                ZktAlarmRecordDO zktAlarmRecordDO = StringUtil.tranferItemToDTO(this.alarmRecord, ZktAlarmRecordDO.class);
                //立即过期，过期的时候可能还没有报警记录ID,需要重新执行下
                String alarmId = alarmRecordRepository.findById(zktAlarmRecordDO.getDefinitionId()).orElse(new ZktAlarmRecordDO()).getAlarmId();
                if(StringUtil.isEmpty(alarmId)) {
                    log.info("refire:",refire);
                    mergedJobDataMap.put("refire",String.valueOf(StringUtil.getInt(refire)+1));
                    reFireJob(context,mergedJobDataMap);
                }
                log.info("报警参数为：",zktAlarmRecordDO.toString());
                NettyMessage<JSONObject> nettyMessage = new NettyMessage<>();
                nettyMessage.setStreamId(nums.getAndIncrement());
                nettyMessage.setOpCode(6);
                JSONObject result = new JSONObject();
                result.put("id",zktAlarmRecordDO.getAlarmId());
                result.put("state","3");
                result.put("groupCode",groupCode);
                result.put("projectId",projectId);
                nettyMessage.setContent(Arrays.asList(result));
                //{"id","123", "state":1, "groupCode":"wd", "projectId":"Pj123"}
                groupNettyClient.sendMessage(nettyMessage.toString());
                //已经过期的时候删除掉这条报警定义了，保证不会再次产生报警
                AlarmState alarmState = new AlarmState(defineId);
                alarmInfoCache.setAlarmState(defineId, alarmState);
                if(alarmRecordRepository.existsById(zktAlarmRecordDO.getDefinitionId())){
                    alarmRecordRepository.deleteById(zktAlarmRecordDO.getDefinitionId());
                }
            }
            log.info("----------------结束---------------------");
        } catch (Exception e) {
            log.error("job hander error",e);
        }

    }


    private void reFireJob(JobExecutionContext context, JobDataMap mergedJobDataMap) {
        try {
            log.info("refire:",refire);
            if (StringUtil.getInt(refire)>1) {
               log.error("重新执行依然失败");
            }else {
                Scheduler scheduler = context.getScheduler();
                Trigger trigger = context.getTrigger();
                DateTime dateTime = DateUtil.offsetSecond(new Date(), 300);
                DateBuilder.IntervalUnit unit = null;
                    // 减去一个提前发单时间
                Trigger newTrigger = TriggerBuilder.newTrigger()
                        .startAt(dateTime.toJdkDate())
                        .withIdentity(trigger.getKey())
                        .usingJobData(mergedJobDataMap)
                        .withSchedule(simpleSchedule().withMisfireHandlingInstructionIgnoreMisfires()).build();
                Date rescheduleJob = scheduler.rescheduleJob(trigger.getKey(), newTrigger);
            }
        } catch (Exception e) {
            log.error("获取不到报警记录ID.重新获取报错！", e);
        }
    }
    public String getAlarmRecord() {
        return alarmRecord;
    }

    public void setAlarmRecord(String alarmRecord) {
        this.alarmRecord = alarmRecord;
    }

    public String getRefire() {
        return refire;
    }

    public void setRefire(String refire) {
        this.refire = refire;
    }

    public String getExpireTime() {
        return expireTime;
    }

    public void setExpireTime(String expireTime) {
        this.expireTime = expireTime;
    }

    public String getDefineId() {
        return defineId;
    }

    public void setDefineId(String defineId) {
        this.defineId = defineId;
    }
}
